#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include <iiimcf.h>
#include "keycodes.h"
#include "im.h"

#ifdef USE_TII
#include "../tii.h"
#endif

int
tii_iiimcf_init(void)
{
	IIIMF_status st;
	IIIMCF_input_method im;

	st = iiimcf_initialize(IIIMCF_ATTR_NULL);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	if(create_handle("Terminal Input Interface"))
		return 2;

	if(register_iiimcf_components())
		return 3;

	if(!(im = get_input_method_id()))
		return 4;

	if(create_context(im))
		return 5;

	/* May move to 'tii_sendkey' if needed */
	if(forward_im_trigger(1))
		return 6;

	return 0;
}

int
tii_iiimcf_sendkey(int key, int shift_state)
{
	int keycode, keychar, mod;

	if(key < 128)
		keycode = keychar_to_keycode[key];
	else
		keycode = IIIMF_KEYCODE_UNDEFINED;
	keychar = key;
	mod = shift_state;

	if(forward_key(keycode, keychar, mod))
		return 1;

	if(dispatch_incoming_events())
		return 2;

	return 0;
}

int
tii_iiimcf_exit(void)
{
	IIIMF_status st;
	IIIMCF_context * c = global_iiimcf_context((IIIMCF_context *)NULL);
	IIIMCF_handle * handle = global_iiimcf_handle((IIIMCF_handle *)NULL);

	if(forward_im_trigger(0))
		return 1;

	st = iiimcf_destroy_context(*c);
	if(st != IIIMF_STATUS_SUCCESS)
		return 2;

	st = iiimcf_destroy_handle(*handle);
	if(st != IIIMF_STATUS_SUCCESS)
		return 3;

	st = iiimcf_finalize();
	if(st != IIIMF_STATUS_SUCCESS)
		return 4;

	return 0;
}

#ifdef USE_TII
/*
 * TII Defaults to IIIMF
 */
IMList im_default = {"default", 
			tii_iiimcf_init,
			tii_iiimcf_sendkey,
			tii_iiimcf_exit};
#endif
